// The PGM/PPM code is old code from a long forgotten source.

#include <stdio.h>
#include <ctype.h>

#include "imageClass.h"

static void warn(char *message)
{
	fprintf(stderr, "pgm/ppm: Error - %s\n", message);
}

bool SavePGM(ImageClassMono &src, const char *filename)
{
	FILE *fp = fopen(filename, "wb");
	if (!fp) 
        {
            printf("cannot open file %s for writing\n", filename);
            return false;
        }
	else
        {
            const int inc = src.getWidth();
            int w=src.getWidth();
            int h=src.getHeight();

            unsigned char *tmp=src.getInternalPointer();

            fprintf(fp, "P5\n%d %d\n%d\n", w, h, 255);
            for (int i = 0; i < h; i++)
                {
                    fwrite((void *) tmp, 1, (size_t) w, fp);
                    tmp += inc;
                }

            fclose(fp);
        }

	return true;
}

bool SavePPM(ImageClassRgb &src, const char *filename)
{
	FILE *fp = fopen(filename, "wb");
	if (!fp) 
        {
            printf("cannot open file %s for writing\n", filename);
            return false;
        }
	else
        {
            const int inc = 3*src.getWidth();
            int w=src.getWidth();
            int h=src.getHeight();

            unsigned char *tmp=src.getInternalPointer();

            fprintf(fp, "P6\n%d %d\n%d\n", w, h, 255);
            for (int i = 0; i < h; i++)
                {
                    fwrite((void *) tmp, 1, (size_t) (w*3), fp);
                    tmp += inc;
                }

            fclose(fp);
        }

	return true;
}


static bool ReadHeader(FILE *fp, int *height, int *width, int *color)
{
	char ch;
	int  maxval;

	*color = 0;

	if (fscanf(fp, "P%c\n", &ch) != 1 || (ch!='6'&&ch!='5')) 
        {
            warn("file is not in pgm/ppm raw format; cannot read");
            return false;
        }

	if (ch=='6') *color = 1;

	// skip comments
	ch = fgetc(fp);
	while (ch == '#')
        {
            do 
                {
                    ch = fgetc(fp);
                } 
            while (ch != '\n');   
            ch = fgetc(fp);    
        }

	while(!isdigit(ch))
        {
            ch = fgetc(fp);
        }

	if (!isdigit(ch)) 
        {
            warn("cannot read header information from pgm/ppm file");
            return false;
        }
	ungetc(ch, fp);

	fscanf(fp, "%d%d%d", width, height, &maxval);
	fgetc(fp);
	if (maxval != 255)
        {
            //die("image is not true-color (24 bit); read failed");
            warn("image is not true-color (24 bit); read failed");
            return false;
        }

	return true;
}

bool ReadPPM(ImageClassRgb &img, const char *filename)
{
   	int width, height, color, num, size;
    FILE *fp=0;
	fp = fopen(filename, "rb");

    if(fp==0)
        {
            fprintf(stderr, "Error opening %s, check if file exists.\n", filename);
            return false;
        }

	if (!ReadHeader(fp, &height, &width, &color))
        {
            fclose (fp);
            fprintf(stderr, "Error reading header, is file a valid ppm/pgm?\n");
            return false;
        }
    if (!color)
        {
            fclose(fp);
            fprintf(stderr, "File is grayscale, conversion not yet supported\n");
            return false;
        }
	
    img.resize(height, width);

	const int w = img.getWidth();
	const int h = img.getHeight();
	const int pad = 3*w;
	unsigned char *dst = img.getInternalPointer ();
	size = w * h;

	num = 0;
	for (int i = 0; i < h; i++)
        {
            num += fread((void *) dst, 1, (size_t) 3*w, fp);
            dst += pad;
        }
    
	fclose(fp);

	return true;
}

bool ReadPGM(ImageClassMono &img, const char *filename)
{
   	int width, height, color, num, size;
    FILE *fp=0;
	fp = fopen(filename, "rb");

    if(fp==0)
        {
            fprintf(stderr, "Error opening %s, check if file exists.\n", filename);
            return false;
        }

	if (!ReadHeader(fp, &height, &width, &color))
        {
            fclose (fp);
            fprintf(stderr, "Error reading header, is file a valid ppm/pgm?\n");
            return false;
        }
    if (color)
        {
            fclose(fp);
            fprintf(stderr, "File is color, conversion not yet supported\n");
            return false;
        }
	
    img.resize(height, width);

	const int w = img.getWidth();
	const int h = img.getHeight();
	const int pad = w;
	unsigned char *dst = img.getInternalPointer ();
	size = w * h;

	num = 0;
	for (int i = 0; i < h; i++)
        {
            num += fread((void *) dst, 1, (size_t) w, fp);
            dst += pad;
        }

	fclose(fp);

    return true;
}

#include <fstream>
#include <string.h>

using namespace std;

//costruttore di default
ImageClassRgb::ImageClassRgb():
width(0),
height(0),
image(0)
{
}

//costruttore n_righe x n_colonne
ImageClassRgb::ImageClassRgb(int r, int c)
{
	width=c;
	height=r;
 
    image=new unsigned char[3*width*height];
}

//distuttore
ImageClassRgb::~ImageClassRgb()
{
	if (image != 0)
		delete[] image;
}

// ridimensiona loggetto
void ImageClassRgb::resize(int r, int c)
{
	if(image!=0)
		delete[] image;

	image = new unsigned char [3*r*c];
	width=c;
	height=r;
}

int ImageClassRgb::getHeight()
{
	return height;
}

int ImageClassRgb::getWidth()
{
	return width;
}

//funzioni di set
void ImageClassRgb::setHeight(int h)
{
	resize(h,width);
}

void ImageClassRgb::setWidth(int w)
{
	resize(height,w);
}


//costruttore di default
ImageClassMono::ImageClassMono():
width(0),
height(0),
image(0)
{
}

//costruttore n_righe x n_colonne
ImageClassMono::ImageClassMono(int r, int c)
{
	width=c;
	height=r;
 
    image=new unsigned char[width*height];
}

//distuttore
ImageClassMono::~ImageClassMono()
{
	if (image != 0)
		delete[] image;
}

// ridimensiona loggetto
void ImageClassMono::resize(int r, int c)
{
	if(image!=0)
		delete[] image;

	image = new unsigned char [r*c];
	width=c;
	height=r;
}

int ImageClassMono::getHeight()
{
	return height;
}

int ImageClassMono::getWidth()
{
	return width;
}

//funzioni di set
void ImageClassMono::setHeight(int h)
{
	resize(h,width);
}

void ImageClassMono::setWidth(int w)
{
	resize(height,w);
}

